# Copyright (c) 2019-2020 Farsight Security, Inc.
# All Rights Reserved
#
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF FARSIGHT SECURITY, INC.
# The copyright notice above does not evidence any
# actual or intended publication of such source code.
from dnstli2.log import *

class HTTPResponse(Exception):
    __slots__ = ('status', 'headers', 'message',)
    def __init__(self, status, message):
        self.status = status
        if message == None:
            self.message = None
        else:
            self.message = (message + '\n').encode('utf-8')
        self.headers = [('Content-Type', 'text/plain')]

Raw200Status = '200 OK'
Raw401Status = '401 unauthorized'
Raw429Status = '429 too many requests'

AuthSuccess = HTTPResponse(
    Raw200Status, 'Success: Authentication succeeded')

BadRequest = HTTPResponse(
    '400 Bad Request', 'Error: unable to parse request')
QuotaExpired = HTTPResponse(
    Raw401Status, 'Error: Quota is expired')

MissingAPIKey = HTTPResponse(
    '403 forbidden', 'Error: X-API-Key header not present')
BadAPIKey = HTTPResponse(
    '403 forbidden', 'Error: Bad API key')
RateLimitExceeded = HTTPResponse(
    Raw429Status, 'Error: Rate limit exceeded')
BurstRateExceeded = HTTPResponse(
    Raw429Status, 'Error: Burst window rate limit exceeded')

RemoteAddressUnauthorized = HTTPResponse(
    '403 forbidden', 'Client IP address not authorized for account')
NoRateLimits = HTTPResponse(
    '403 forbidden', 'Error: Rate Limits are not supported')

InternalServerError = HTTPResponse(
    '500 internal server error', 'Error: exception occurred')

APIVersionMismatch  = HTTPResponse(
    '401 unauthorized', 'Error: Your API key may not query this API version')

UnsupportedMediaType = HTTPResponse(
    '415 Unsupported Media Type', 'Error: The Accept: header does not specify a supported content type for this query')

OffsetOverMax = HTTPResponse(
    '416 Requested Range Not Satisfiable', 'Error: offset value greater than maximum allowed')
OffsetDisallowed = HTTPResponse(
    '416 Requested Range Not Satisfiable', 'Error: offset not allowed for this type of API key')
OffsetUnsupported = HTTPResponse(
    '416 Requested Range Not Satisfiable', 'Error: offset not allowed for this type of query')

cond_begin = 'begin'
cond_ongoing = 'ongoing'
cond_succeeded = 'succeeded'
cond_limited = 'limited'
cond_failed = 'failed'

def format_saf_row(cond = None, obj = None, msg = None):
    row = '{'
    if cond:
        row += '"cond":"' + cond + '"'
        if obj or msg:
            row += ','
    if obj:
        row += '"obj":' + obj
        if msg:
            row += ','
    if msg:
        row += '"msg":"' + msg + '"'

    row += '}\n'
    return row

pingOk = '{"ping": "ok"}\n'.encode('utf-8')


# Which content types are allowed for lookup and summarize queries
allowed_query_content_types = {
    'application/ldjson',
    'application/x-ldjson',
    'application/ndjson',
    'application/x-ndjson',
    'application/jsonl',
    'application/x-jsonl' }

# The preferred content type if */* is requested
preferred_query_content_type = 'application/x-ndjson'

def content_type_to_return(env, allowed_cts, preferred_ct):
    '''
    Check if the HTTP_ACCEPT header in env contains one of the allowed content types.
    Return the first matching one.
    If find the wild card */* then use the preferred content type.
    Raises UnsupportedMediaType if none of the content types are allowed.
    '''
    if 'HTTP_ACCEPT' in env:
        # extract content types acceptable from HTTP request
        ctypes = env['HTTP_ACCEPT'].split(',')
        # for each, lower case it and strip out any priority field
        for ctplus in ctypes:
            ct = ctplus.strip().lower().split(';', 1)[0]
            if ct == "*/*":
                return preferred_ct
            if ct in allowed_cts:
                return ct
        log(DEBUG, "Saw only unsupported values in HTTP_ACCEPT "  + str(ctypes))
    else:
        log(DEBUG, "Request does not include a HTTP_ACCEPT header")
    raise UnsupportedMediaType
