# `dnstli2` command-line tools

This suite of tools contains the following tools:

 * `dnstli2_report`: dnstable report tool

# `dnstli2` source installation
Note that `dnstli2` is only supported under Linux.

`dnstli2` uses Python 3.  Use the provided installation script to
install `dnstli2` command-line programs and libraries:

`$ sudo python3 ./setup.py build install`.

# `dnstli2` configuration

`dnstli2` is designed to run inside a uwsgi container.

A typical command-line for that is:
```
/usr/bin/uwsgi --plugin syslog --plugin python35 --ini /etc/dnstli/dnstli2.ini
```

## `dnstli2.ini`

This is the uwsgi configuration file, typically located in `/etc/dnstli/dnstli2.ini`.
Here is a sample one that specifies "apikey" based authentication:
```
[uwsgi]
module = dnstli2.uwsgi_app:apikey
socket = /run/dnstli2/uwsgi-api.socket
stats = /run/dnstli2/uwsgi-api-stats.socket
uid = dnstli
gid = dnstli
chmod = 777

logger = syslog:uwsgi_dnstli2_api
disable-logging = 1
log-micros = 0
ignore-sigpipe = 1
ignore-write-errors = 1
disable-write-exception = 1

need-app = 1
die-on-term = 1
enable-threads = 0
master = 1
thread = 0
# increase this if you expect a heavy load; lower for a lightly used system
processes = 25

# after max-requests are served, restart the worker process
max-requests = 2000
# after 86400 seconds (24 hours), restart the worker process
max-worker-lifetime = 86400

# Before you increase "listen", make sure you set
#    # sysctl net.core.somaxconn=1024
#    # echo net.core.somaxconn = 1024 >>/etc/sysctl.conf
#listen = 1024
listen = 128

# 900 seconds = 15 minutes
harakiri = 900

auto-procname = 1
procname-prefix-spaced = [DNSDB_APIv2]
```

To use "noauth" authentication instead, change
```
module = dnstli2.uwsgi_app:apikey
```
to
```
module = dnstli2.uwsgi_app:noauth
```
If you do that, do not provide the `X-API-Key` HTTP header in your
requests, and the `/dnsdb/v2/rate_limit` API endpoint will not exist.

## `dnstli2.conf`


`dnstli2` is configured using a json-formatted file. The default location
for this file is `/etc/dnstli/dnstli2.conf`. This can be overridden by changing
the `DNSTLI_CONFIG_FILE` environment variable.

Required configuration:

| Key | Notes |
| --- | ----- |
| `filesets` | List of filesets.  Each is an object with keys `setfile` (path to file), `iszone` (boolean), and `isdnssec` (boolean). |
| `res_limit_default` | Default result limit. Integer number of rows. |
| `res_limit_max` | Maximum result limit. Integer number of rows. |

Optional configuration:

| Key | Notes |
| --- | ----- |
| `admin_addresses` | List of addresses that may connect as any account.  Default is to allow any address to connect. |
| `export_installation` | Boolean if a DNSDB Export installation.  If true, disallows the query\_limit\_redis\_* parameters and other changes. |
| `query_limit_default` | Default number of queries per user per period.  Required if the redis parameters are set. |
| `query_limit_period` | Query limit period in seconds.  Required if the redis parameters are set.|
| `query_limit_redis_socket` | Path to redis socket for query limit tracking.  Default is to not use redis. |
| `query_limit_redis_url` | URI to redis server for query limit tracking.  Default is to not use redis. |
| `query_exit_check_interval` | How often to interrupt the dnstable query to check for client disconnection (miliseconds).  Default is 0.25 seconds, which is reasonable in most cases.  |
| `max_query_time` | How many seconds maximum a query can run.  Default is unlimited.  It only applies after the first row and between rows the query is streaming back.  While dnstli2 is still waiting for a result from the dnstable library then max_query_time is not applicable. |
| `offset_max` | Maximum offset allowed. Integer number of rows. Default is very large. |
| `heartbeat_interval` | How many seconds between dnstli2 sending a SAF heartbeat response while awaiting results. If not defined, heartbeats will not be sent. |
| `cors_origins` | List of valid Origins for HTTP CORS.  Default is '*', which means all origins are valid. |
| `wildcard_label` | Label for wildcard summary records. Default is None, which means wildcard summary records may not be queried directly but may appear in the results of wildcard queries. |

-------------------------------------------------------------------------------

# Cross-Origin Resource Sharing (CORS)

This section is only meaningful for web browser-based clients and does not
apply to programs such as `dnsdbq`.

`dnstli2` 4.x has support for [Cross Origin Resource Sharing]. `dnstli2` uses
a CORS policy of `*` unless you set the `cors_origins` configuration option.
This option is a list of strings, each of which is an Origin as per the
below-linked specification. If the browser does not provide an Origin header
this feature is disabled.

[Cross Origin Resource Sharing]: https://www.w3.org/TR/cors

Example:

```json
{
    "cors_origins": [
        "https://dnsdb.info"
    ]
}
```

-------------------------------------------------------------------------------

# dnstli2_report
This is the FSI `dnstli2` report tool. It is a command-line tool used to parse an
FSI dnstli logfile and return collated user-centric statistics. Depending on
options specified, the output will be suitable for human consumption or
machine-readable for use with other automation.  After showing some
examples of the raw tool, we show how gnuplot can be used to generate
some graphs.

dnstli2_report requires Python version 3.x.


## dnstli2_report examples
`dnstli2_report` can be invoked in one of four operational modes and three
output formats.  In the examples below "dnsdb.notice-100000" is file
with the last 100000 lines from /var/log/dnsdb/dnsdb.notice, the
standard location for the dnstli log files.

~~~
usage: dnstli2_report [-h] [-H] [-c {qpd,qpw,qpm,upd,upw,upm,all}] [-m [MONTH]]
                     [-d [DAYS]] [-o {text,csv,json}] [-f] [-u USER] [-v] [-V]
                     [-y YEAR]
                     logfile

dnstli2 report tool

positional arguments:
  logfile               dnstli logfile

optional arguments:
  -h, --help            show this help message and exit
  -H, --header          print header row (CSV output only)
  -c {qpd,qpw,qpm,upd,upw,upm,all}, --counts {qpd,qpw,qpm,upd,upw,upm,all}
                        generate query count report (across all dates/months).
                        Specify which reports are generated as: qpd=Queries Per
                        Date, qpw=Queries Per Week, qpm=Queries Per Month,
                        upd=Unique users per Date, upw=Unique users per Week,
                        upm=Unique users per Month, all=All of the above
  -m [MONTH], --month [MONTH]
                        generate monthly report for all users for month (1-12)
  -d [DAYS], --days [DAYS]
                        for how many days from today to generate a daily
                        report.
  -o {text,csv,json}, --output {text,csv,json}
                        output format (unstructured text, CSV, or newline
                        delimited JSON)
  -f, --full_report     generate full report
  -u USER, --user USER  generate report for single user
  -v, --verbose         verbose output
  -V, --version         show program's version number and exit
  -y YEAR, --year YEAR  set starting year for logfile if the log file time
                        format does not include the year
~~~

For each of the first three modes, the following information will be emitted per user:

 * `user`: username
 * `count`: number of times seen
 * `last seen on`: ISO 8601 timestamp of when user was last seen

For additional output in any of these three modes, use the `-v`
(verbose) switch to print more information:

 * `total time`: time it took to retrieve the results from the data store
 * `total bytes`: total number of bytes returned for user
 * `total results`: total number of results returned for user
 * `ips`: list of IP addresses user sourced queries from

### User Report Mode
`--user|-u username`

Query statistics for a specific user. Output as text:

~~~
$ dnstli2_report -u userXYZ dnsdb.notice-100000
user: userXYZ
count: 161
last seen on: 2014-10-10T02:42:28Z
~~~

Output as text (verbose):

~~~
$ dnstli2_report -v -u userXYZ dnsdb.notice-100000
user: userXYZ
count: 161
last seen on: 2014-10-10T02:42:28Z
total time: 11.69
total bytes: 12664837
total results: 14332
ips: 192.168.1.121,10.1.1.101
~~~

Output as CSV:

~~~
$ dnstli2_report -u userXYZ dnsdb.notice-100000 -o csv -H
user,count,last seen on
userXYZ,161,2014-10-10T02:42:28Z
~~~

Output as CSV (verbose):

~~~
$ dnstli2_report -v -u userXYZ dnsdb.notice-100000 -o csv -H
user,count,last seen on,total time,total results,total bytes,ips
userXYZ,161,2014-10-10T02:42:28Z,11.69,14332,12664837,"192.168.1.121,10.1.1.101"
~~~

Output as JSON:

~~~
$ dnstli2_report -u userXYZ dnsdb.notice-100000 -o json
{"count": 161, "last seen on": "2014-10-10T02:42:28Z", "user": "userXYZ"}
~~~

Output as JSON (verbose):

~~~
$ dnstli2_report -v -u userXYZ dnsdb.notice-100000 -o json
{"count": 161, "total time": 11.689999999999982, "ips": "192.168.1.121,10.1.1.101", "user": "userXYZ", "total results": 14332, "last seen on": "2014-10-10T02:42:28Z", "total bytes": 12664837}
~~~

### Monthy Report Mode
`--month|-m [numeric_month]`

Query statistics for a full month (defaults to current month) and emit number
of queries per user and timestamp last seen. Output as text:

~~~
$ dnstli2_report -m10 dnsdb.notice-100000
monthly report for October
user: userDEF
count: 1131
last seen on: 2014-10-10T01:23:47Z

user: userABC
count: 23
last seen on: 2014-10-10T02:46:42Z

user: user123
count: 5
last seen on: 2014-10-09T20:43:58Z

...
~~~

Output as CSV:

~~~
$ dnstli2_report -m10 dnsdb.notice-100000 -o csv -H
user,count,last seen on
userDEF,1131,2014-10-10T01:23:47Z
userABC,23,2014-10-10T02:46:42Z
user123,5,2014-10-09T20:43:58Z
...
~~~

Output as JSON:

~~~
$ dnstli2_report -m10 dnsdb.notice-100000 -o json
{"count": 1131, "last seen on": "2014-10-10T01:23:47Z", "user": "userDEF"}
{"count": 23, "last seen on": "2014-10-10T02:46:42Z", "user": "userABC"}
{"count": 5, "last seen on": "2014-10-09T20:43:58Z", "user": "user123"}
...
~~~

### Full Report Mode
`--full_report|-f`

Query statistics across entire log file. This is the default behavior. Output
as text:

~~~
$ dnstli2_report dnsdb.notice-100000
user: userDEF
count: 1131
last seen on: 2014-10-10T01:23:47Z

user: userABC
count: 23
last seen on: 2014-10-10T02:46:42Z

user: user123
count: 5
last seen on: 2014-10-09T20:43:58Z

...
~~~

Output as CSV:

~~~
$ dnstli2_report dnsdb.notice-100000 -o csv -H
user,count,last seen on
userDEF,1131,2014-10-10T01:23:47Z
userABC,23,2014-10-10T02:46:42Z
user123,5,2014-10-09T20:43:58Z
...
~~~

Output as JSON:

~~~
$ dnstli2_report dnsdb.notice-100000 -o json
{"count": 1131, "last seen on": "2014-10-10T01:23:47Z", "user": "userDEF"}
{"count": 23, "last seen on": "2014-10-10T02:46:42Z", "user": "userABC"}
{"count": 5, "last seen on": "2014-10-09T20:43:58Z", "user": "user123"}
...
~~~

### Counts Report Mode
`--counts|-c`

Generate a query counts report (across all months), across entire log file.  Specify which reports are generated as:

  * qpd: Queries Per Date
  * qpw: Queries Per Week
  * qpm: Queries Per Month
  * upd: Unique users per Date
  * upw: Unique users per Week
  * upm: Unique users per Month
  * all: All of the above

~~~
$ dnstli2_report -v -o json -c all -y 2018 -d 30 input-log
{"Queries per date": {"2019-10-27": 7, "2019-11-01": 3, "2019-11-19": 1, "2019-11-20": 1, "2019-11-21": 8}}
{"Queries per week": {"2019-01": 3, "2019-43": 7, "2019-44": 3, "2019-47": 10}}
{"Queries per month": {"2018-12": 1, "2019-01": 2, "2019-10": 7, "2019-11": 13}}
{"Unique users per date": {"2019-10-27": 1, "2019-11-01": 1, "2019-11-19": 1, "2019-11-20": 1, "2019-11-21": 1}}
{"Unique users per week": {"2019-01": 3, "2019-43": 1, "2019-44": 1, "2019-47": 1}}
{"Unique users per month": {"2018-12": 1, "2019-01": 2, "2019-10": 1, "2019-11": 2}}
Results from 2018-12-31T14:27:23 to 2019-11-21T14:27:23
~~~

The -d days parameter controls how many days back for the qpd and upd reports.  Here is an example showing going back 30 days.

~~~
$ dnstli2_report -v -o csv -c qpd -d 30 input2-log
Queries per date,
2019-10-27,7
2019-11-01,3
2019-11-19,1
2019-11-20,1
2019-11-21,8

Results from 2019-01-02T17:30:23 to 2019-11-21T14:27:23
~~~
If we redo that report for just 7 days:

~~~
$ dnstli2_report -v -o csv -c qpd -d 7 in2
Queries per date,
2019-11-19,1
2019-11-20,1
2019-11-21,8

Results from 2019-01-02T17:30:23 to 2019-11-21T14:27:23
~~~

## Plotting the results

Install gnuplot 4.2 or higher, which is in standard distributions of Debian and Redhat/CentOS.

`# yum install gnuplot`
or
`# apt-get install gnuplot`


Save verbose CSV outputs to file `dnstli_output.csv` without headers:
~~~
$ dnstli2_report dnsdb.notice-100000 -o csv -v > dnstli_output.csv
~~~

Run gnuplot with either of the two provided scripts:

### `gnuplot plot-to-qt.gnuplot`

If you want view the graph in qt, which can be useful for making modifications.

### `gnuplot plot-to-png.gnuplot`
If you want to produce a PNG file with the plot, `DNSDB_Usage.png`.
